package Team5.SmartTowns;

import org.junit.jupiter.api.*;
//import org.junit.jupiter.api.BeforeAll;
//import org.junit.jupiter.api.BeforeEach;


import org.junit.After;
import org.junit.Before;


//import org.junit.Test; //- using this stops initialisation error, and runs properly, but breaks firefox driver
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.runner.RunWith;

import org.openqa.selenium.By;
import org.openqa.selenium.NoAlertPresentException;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.firefox.FirefoxDriver;
import org.openqa.selenium.firefox.FirefoxOptions;
import org.openqa.selenium.support.ui.Select;
import org.openqa.selenium.support.ui.WebDriverWait;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.Profile;
import org.springframework.test.annotation.DirtiesContext;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.context.junit4.SpringRunner;

import io.github.bonigarcia.wdm.WebDriverManager;


import java.time.Duration;

import static javax.swing.text.html.CSS.getAttribute;
import static org.bouncycastle.oer.its.ieee1609dot2.basetypes.Duration.seconds;
import static org.junit.Assert.assertTrue;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;

//@RunWith(SpringRunner.class)
@ExtendWith(SpringExtension.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
//system and functional testing.
//@ActiveProfiles("test")
public class FirefoxDriverPagesTests {
    @Value("${local.server.port}")
    private int port;

    WebDriver webDriver;

    @BeforeAll // setupFirefoxDriver
    static void setupClass() {
        WebDriverManager.firefoxdriver().setup();
    }

    @BeforeEach
    void setupTest() {
        FirefoxOptions options = new FirefoxOptions();
        options.addArguments("--headless");
        webDriver = new FirefoxDriver(options);
//        webDriver = new FirefoxDriver();
    }
    @AfterEach
    void teardown() {
        webDriver.quit();
    }


    @Test
    public void pageSetupTest() {
    }

    @Test
    public void testHomePageContents() {
        this.webDriver.get("http://localhost:" + Integer.toString(port) + "/mobile-home");
//        assertEquals(1,1);
        assertTrue(webDriver.findElement(By.cssSelector("main")).getText().contains("Welcome to VZTA Smart Towns!"));


    }
//test false entries, log ina dn log out?, test Risca and Penarth at least once

    @Test
   @DirtiesContext
    @Order(1)
    public void testingLandmarkSubmissionErrorAlertsShow(){
        this.webDriver.get("http://localhost:" + Integer.toString(port) + "/landmarkSubmission");
        //check error messages aren't showing straight away
        assertFalse(webDriver.findElement(By.cssSelector("main form")).getText().contains("You must type in a username."));
        assertFalse(webDriver.findElement(By.cssSelector("main form")).getText().contains("You must attach a contact address.")); //css selector?
        assertFalse(webDriver.findElement(By.cssSelector("main form")).getText().contains("You must attach a valid contact address.")); //css selector?


        //check submitting empty form.
        assertTrue(webDriver.findElement(By.cssSelector("main form ")).getText().contains("Submit"));
        this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click();
//        this.webDriver.wait(200);
        //WebDriverWait wait = new WebDriverWait(this.webDriver, Duration.ofSeconds(100) /*timeout in seconds*/);
        assertTrue(webDriver.switchTo().alert().getText().contains("Invalid trail selected. \nPlease select the trail you wish to join."));
        this.webDriver.switchTo().alert().accept();
        assertTrue(webDriver.switchTo().alert().getText().contains("Invalid location selected. \nPlease select the location you wish to join."));
        this.webDriver.switchTo().alert().accept();
        //testing location-trail mismatch
        new Select(this.webDriver.findElement(By.name("landmarkLocation"))).selectByVisibleText("Risca");
        new Select(this.webDriver.findElement(By.name("trailID"))).selectByVisibleText("(Penarth) Esplanade Trail");
        this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click();
        assertTrue(webDriver.switchTo().alert().getText().contains("Trail unavailable in your current location. \nPlease choose an available trail"));
        this.webDriver.switchTo().alert().accept();
        new Select(this.webDriver.findElement(By.name("landmarkLocation"))).selectByVisibleText("Penarth"); //change location to be acceptable
        this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click(); //submit to show backend verification.
        assertTrue(webDriver.findElement(By.xpath("/html/body/main/form/div[1]")).getText().contains("You must type in a username."));
        assertTrue(webDriver.findElement(By.xpath("/html/body/main/form/div[2]")).getText().contains("You must attach a contact address.")); //css selector?
        new Select(this.webDriver.findElement(By.name("landmarkLocation"))).selectByVisibleText("Penarth");
        //testing description word count
        this.webDriver.findElement(By.name("landmarkDescription")).sendKeys("This is a text paragraph to test this form’s 200-word limit. This is a text paragraph to test this form’s 200-word limit." +
                " This is a text paragraph to test this form’s 200-word limit. This is a text paragraph to test this form’s 200-word limit. This is a text paragraph to test this form’s 200-word limit. This is a text " +
                "paragraph to test this form’s 200-word limit. This is a text paragraph to test this form’s 200-word limit. This is a text paragraph to test this form’s 200-word limit. This is a text paragraph to test this " +
                "form’s 200-word limit. This is a text paragraph to test this form’s 200-word limit. This is a text paragraph to test this form’s 200-word limit. This is a text paragraph to test this form’s 200-word limit. This is a " +
                "text paragraph to test this form’s 200-word limit. This is a text paragraph to test this form’s 200-word limit. This is a text paragraph to test this form’s 200-word limit. This is a text paragraph to test this form’s 200-word limit. " +
                "This is a text paragraph to test this form’s 200-word limit. This is a text paragraph to test this form’s 200-word limit. Three more words.");
        this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click();
        assertTrue(webDriver.switchTo().alert().getText().contains("Please keep your description to a maximum of 200 words."));
        this.webDriver.switchTo().alert().accept();
        this.webDriver.findElement(By.name("landmarkDescription")).clear();
        this.webDriver.findElement(By.name("landmarkDescription")).sendKeys("This is a mock description under 200 words");

        this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click();
        boolean alertPresent; //make sure alert doesnt show for correct trail
        try{
            this.webDriver.switchTo().alert();
            alertPresent=true;

        } catch (NoAlertPresentException Ex){
            alertPresent=false;
        }
        assertFalse(alertPresent);
        //testing javascript verification output
        this.webDriver.findElement(By.name("landmarkEmail")).sendKeys("MockAddressNotEmail");
        this.webDriver.findElement(By.name("landmarkName")).sendKeys("MockLocationWorksTMP");
        this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click();

        assertFalse(webDriver.findElement(By.cssSelector("main form")).getText().contains("You must type in a username."));
        assertFalse(webDriver.findElement(By.cssSelector("main form")).getText().contains("You must attach a contact address.")); //css selector?
        assertTrue(webDriver.findElement(By.xpath("/html/body/main/form/div")).getText().contains("You must attach a valid contact address.")); //css selector?
        this.webDriver.findElement(By.name("landmarkEmail")).clear();
        this.webDriver.findElement(By.name("landmarkEmail")).sendKeys("MockAddress@Works.co.uk");
        //make sure all error messages disappear once fulfilled
        this.webDriver.findElement(By.name("landmarkName")).clear();
        this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click();
        assertFalse(webDriver.findElement(By.cssSelector("main form")).getText().contains("You must attach a valid contact address.")); //css selector?
        this.webDriver.findElement(By.name("landmarkName")).sendKeys("MockLocationWorks");
        this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click();
        assertTrue(webDriver.getCurrentUrl().contains("http://localhost:" + Integer.toString(port) + "/mobile-home"));
    }

    @Test
    @DirtiesContext(methodMode = DirtiesContext.MethodMode.AFTER_METHOD)
    @Order(2)
    public void testingAdminAcceptanceForm(){
        this.webDriver.get("http://localhost:" + Integer.toString(port) + "/login");
        this.webDriver.findElement(By.xpath("/html/body/main/div/div[2]/form/label[1]/input")).sendKeys("Admin");
        this.webDriver.findElement(By.xpath("/html/body/main/div/div[2]/form/label[2]/input")).sendKeys("admin");
        this.webDriver.findElement(By.xpath("/html/body/main/div/div[2]/form/button")).click();
        assertTrue(webDriver.getCurrentUrl().contains("http://localhost:" + Integer.toString(port) + "/mobile-home"));
        this.webDriver.get("http://localhost:" + Integer.toString(port) + "/checkpointApproval");
//        assertTrue(webDriver.findElement(By.cssSelector("main")).getText().contains("MockAddress@Works.co.uk"));
        this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click();
        //verify no coord inputs error alert works
        assertTrue(webDriver.switchTo().alert().getText().contains("No location selected. \nPlease select an available location."));
        this.webDriver.switchTo().alert().accept();
        assertTrue(webDriver.switchTo().alert().getText().contains("Invalid location inputted. \nPlease input a valid latitude."));
        this.webDriver.switchTo().alert().accept();
        assertTrue(webDriver.switchTo().alert().getText().contains("Invalid location inputted. \nPlease input a valid longitude."));
        this.webDriver.switchTo().alert().accept();
        //test incorrect coords entered(as per town boundaries todo put method relevant here)
        new Select(this.webDriver.findElement(By.name("locationName"))).selectByVisibleText("St Cenydd (101)");
        this.webDriver.findElement(By.name("locationCoordsLat")).sendKeys("123");
        this.webDriver.findElement(By.name("locationCoordsLong")).sendKeys("123");
        //check user directed to correct error form
        this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click();
        assertTrue(webDriver.getCurrentUrl().contains("http://localhost:" + Integer.toString(port) + "/checkpointApproval?error"));
        assertTrue(webDriver.findElement(By.xpath("/html/body/main/form/div")).getText().contains("Invalid Coordinates entered."));
        new Select(this.webDriver.findElement(By.name("locationName"))).selectByVisibleText("St Cenydd (101)");
        this.webDriver.findElement(By.name("locationCoordsLat")).sendKeys("51.57815");
        this.webDriver.findElement(By.name("locationCoordsLong")).sendKeys("-3.22510");
        this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click();
        //ensure submission submits
        assertTrue(webDriver.getCurrentUrl().contains("http://localhost:" + Integer.toString(port) + "/mobile-home"));



    }


@Test
@DirtiesContext
@Order(3)
public void testingEntireFeature(){
        //homePage
    //todo error if submitted location same as approved location(*in trail) and submitted on approvalform
    this.webDriver.get("http://localhost:" + Integer.toString(port) + "/mobile-home");
    assertTrue(webDriver.findElement(By.cssSelector("main")).getText().contains("Welcome to VZTA Smart Towns!"));
    assertTrue(webDriver.findElements(By.xpath("/html/body/main/div[1]/a[1]/button")).size() > 0);
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/div[1]/a[1]/button")).getText().contains("Submit Landmark"));
    this.webDriver.findElement(By.xpath("/html/body/main/div[1]/a[1]/button")).click();
    //Landmark Sign Up Form
    assertTrue(webDriver.findElement(By.cssSelector("main form h1")).getText().contains("Interested in joining our trails? Sign up Here!"));
    this.webDriver.findElement(By.name("landmarkName")).sendKeys("MockLocation");
    this.webDriver.findElement(By.name("landmarkEmail")).sendKeys("MockAddress@Contact.co.uk");
    this.webDriver.findElement(By.name("landmarkDescription")).sendKeys("This is a mock description of this business.");
    new Select(this.webDriver.findElement(By.name("landmarkLocation"))).selectByVisibleText("Caerphilly");
    new Select(this.webDriver.findElement(By.name("trailID"))).selectByVisibleText("(Caerphilly) Heritage Trail");
    assertTrue(webDriver.findElement(By.cssSelector("main form ")).getText().contains("Submit"));
    assertTrue(webDriver.findElements(By.xpath("/html/body/main/form/button")).size() > 0);
    this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click();
    //return to home page
    assertTrue(webDriver.findElement(By.cssSelector("main div")).getText().contains("Welcome to VZTA Smart Towns!"));
    assertFalse(webDriver.findElement(By.cssSelector("main div")).getText().contains("Review Landmark!"));
    //Log In as Administrator (This user-feature is not what's being tested, as such it won't be covered apart from here)
    assertTrue(webDriver.findElement(By.cssSelector("header nav")).getText().contains("Log In"));
    assertTrue(webDriver.findElement(By.xpath("/html/body/header/nav/ul/li[4]")).getText().contains("Log In"));
//    assertEquals(webDriver.findElement(By.xpath("/html/body/header/nav/ul/li[4]")).getAttribute("href"),("localhost:" + Integer.toString(port) + "/login"));
    assertTrue(webDriver.findElements(By.xpath("/html/body/header/nav/ul/li[4]")).size() > 0);
    this.webDriver.findElement(By.xpath("/html/body/header/nav/ul/li[4]/a")).click();

    //sign in
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/div/div[2]/h1")).getText().contains("Sign In"));
    this.webDriver.findElement(By.xpath("/html/body/main/div/div[2]/form/label[1]/input")).sendKeys("Admin");
    this.webDriver.findElement(By.xpath("/html/body/main/div/div[2]/form/label[2]/input")).sendKeys("admin");
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/div/div[2]/form/button")).getText().contains("SIGN IN"));
    assertTrue(webDriver.findElements(By.xpath("/html/body/main/div/div[2]/form/button")).size() > 0);
    this.webDriver.findElement(By.xpath("/html/body/main/div/div[2]/form/button")).click();

    //check review shows for admin
    assertTrue(webDriver.findElement(By.cssSelector("main div")).getText().contains("Review Landmark!"));
    assertTrue(webDriver.findElements(By.xpath("/html/body/main/div[1]/a[2]")).size() > 0);
    this.webDriver.findElement(By.xpath("/html/body/main/div[1]/a[2]")).click();


    assertTrue(webDriver.findElement(By.xpath("/html/body/main/section/h2")).getText().contains("To Be Approved"));
    assertTrue(webDriver.findElement(By.cssSelector("main h1")).getText().contains("Locations To Be Approved:"));
//todo FIX THIS!!
    assertTrue(webDriver.findElement(By.cssSelector("main table tbody ")).getText().contains("MockLocation (103)"));
    assertTrue(webDriver.findElement(By.cssSelector("main section table tbody")).getText().contains("MockAddress@Contact.co.uk"));
    assertTrue(webDriver.findElement(By.cssSelector("main section table tbody")).getText().contains("This is a mock description of this business."));
    //Can't test Caerphilly;ly as others already there todo? look into Risca
    //todo test verification of this form
    //<!--            values used to be 0103, 0102, intro error becuase the trailnamefromid trailRepoJDBC used string valeus to search DB and not ints
    new Select(this.webDriver.findElement(By.name("locationName"))).selectByVisibleText("MockLocation (103)");
    this.webDriver.findElement(By.name("locationCoordsLat")).sendKeys("51.57815");
    this.webDriver.findElement(By.name("locationCoordsLong")).sendKeys("-3.22510");
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/form/button")).getText().contains("Submit"));
    assertTrue(webDriver.findElements(By.xpath("/html/body/main/form/button")).size() > 0);
    this.webDriver.findElement(By.xpath("/html/body/main/form/button")).click();
    //back to home, check approved variable is gone from approval screen
    assertTrue(webDriver.findElement(By.cssSelector("main")).getText().contains("Welcome to VZTA Smart Towns!"));
    this.webDriver.findElement(By.xpath("/html/body/main/div[1]/a[2]")).click();
    assertFalse(webDriver.findElement(By.cssSelector("main table")).getText().contains("MockLocation (103)"));
    //go back to hiome following header
    assertTrue(webDriver.findElement(By.xpath("/html/body/header/nav/ul/li[1]")).getText().contains("Home"));
    assertTrue(webDriver.findElements(By.xpath("/html/body/header/nav/ul/li[1]")).size() > 0); //todo fix tghis bit
    this.webDriver.findElement(By.xpath("/html/body/header/nav/ul/li[1]/a")).click();
    assertTrue(webDriver.findElement(By.cssSelector("main")).getText().contains("Welcome to VZTA Smart Towns!")); // back home here
    //check town is there, click to go to trails page
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/article/div[1]/h2")).getText().contains("Caerphilly"));
    assertTrue(webDriver.findElements(By.xpath("/html/body/main/article/div[1]/a")).size() > 0);
    this.webDriver.findElement(By.xpath("/html/body/main/article/div[1]/a")).click();
    // check path is in towns trails page
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/section/a[3]/h1")).getText().contains("Caerphilly Heritage Trail"));
    assertTrue(webDriver.findElements(By.xpath("/html/body/main/section/a[3]")).size() > 0);
    this.webDriver.findElement(By.xpath("/html/body/main/section/a[3]")).click();

    //Trail-list page test
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/article[1]")).getText().contains("Caerphilly Heritage Trail"));
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/article[1]/h3")).getText().contains("Checkpoints"));
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/article[1]")).getText().contains("MockLocation"));
    assertTrue(webDriver.findElements(By.xpath("//a[text()='MockLocation']")).size() > 0); //tests MockLocation html name is there
    this.webDriver.findElement(By.xpath("//a[text()='MockLocation']")).click();
    //Trail-location page test
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/article/h1")).getText().contains("MockLocation"));
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/article/h4")).getText().contains("Caerphilly"));
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/article/p")).getText().contains("This is a mock description of this business."));
    assertTrue(webDriver.findElement(By.xpath("/html/body/main/article/iframe")).getAttribute("src").contains("https://maps.google.com/maps?q=51.57815,-3.2251&hl=en&z=20&output=embed"));




}
}
